import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

/**
 * A graphical way to view the map and draw/view the particles.
 * @author Matt Hoffman
 * @version 1.0
 ***/
public class GridMap extends JPanel
{
    private int mRobotX;
    private int mRobotY;
    private float mRobotR;

    private class Particle 
    {
        public Particle(int x, int y) { X=x; Y=y; }
        int X = 0;
        int Y = 0;
    }
    
    private ArrayList<Particle> mParticles;

    public final int maxX, maxY;
    public final boolean[][] map;
    
    /**
     * Construct the grid map given a map-file (which will be
     * provided). 
     * @param file The file specifying the map.
     ***/
    public GridMap(String file, int initX, int initY)
    { 
        mRobotX = initX;
        mRobotY = initY;
        String[] strings = null;
        BufferedReader input = null;
        mParticles = new ArrayList<Particle>();

        try {
            input = new BufferedReader(new FileReader(file));

            // check the version
            if(!(input.readLine()).startsWith("version: 1")) 
                throw new Exception ("Invalid file version! Either " +
                        "your file is wrong, or you should upgrade.");

            // grab the width/height
            strings = (input.readLine()).split("\\s+");
        }
        catch (Exception e) {
            System.out.println("" + e);
            System.exit(-1);
        }

        maxX = Integer.parseInt(strings[0]);
        maxY = Integer.parseInt(strings[1]);
        map  = new boolean[maxY][maxX];

        // build our map
        for (int y=0; y < maxY; ++y)
        {
            try {
                String line = input.readLine();
                for (int x=0; x < maxX; ++x)
                    map[y][x] = line.charAt(x) == '1' ? true : false;
            } 
            catch (IOException e) { System.out.println(""+e); }
        }

        setPreferredSize(new Dimension(maxX, maxY));
    }

    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        for (int y=0; y < maxY; ++y)
            for (int x=0; x < maxX; ++x)
            {
                if (map[y][x]) g.setColor(Color.gray);
                else g.setColor(Color.white);

                // draw the pixel
                g.fillRect(x, y, 1, 1);
            }

        g.setColor(Color.green);
        for (int i=0; i < mParticles.size(); ++i)
        {
            Particle p = mParticles.get(i);
            g.drawOval(p.X, p.Y, 4,4);
            g.fillOval(p.X, p.Y, 4,4);
        }

        g.setColor(Color.red);
        g.drawOval(mRobotX, mRobotY, 8, 8);
        g.fillOval(mRobotX, mRobotY, 8, 8);
    }

    public void setPosition(int x, int y)
    {
        mRobotX = x;
        mRobotY = y;
    }

    /** 
     * Draw a point for some particle at the specific position.
     * @param x The x position.
     * @param y The y position.
     ***/
    public void addParticle(int x, int y) 
    { 
        Particle p = new Particle(x,y);
        mParticles.add(p);
    }

    /** Clear the set of particles that are drawn. */
    public void clearParticles() { mParticles.clear(); }
}
